@@ -1,21 +1,6 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2014-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 """Utilities to show various version information."""
 
@@ -26,36 +11,43 @@ import os.path
 import platform
 import subprocess
 import importlib
-import collections
+import pathlib
+import configparser
 import enum
 import datetime
 import getpass
 import functools
 import dataclasses
-from typing import Mapping, Optional, Sequence, Tuple, cast
+import importlib.metadata
+from typing import (Mapping, Optional, Sequence, Tuple, ClassVar, Dict, Any,
+                    TYPE_CHECKING)
 
-from PyQt5.QtCore import PYQT_VERSION_STR, QLibraryInfo
-from PyQt5.QtNetwork import QSslSocket
-from PyQt5.QtGui import (QOpenGLContext, QOpenGLVersionProfile,
-                         QOffscreenSurface)
-from PyQt5.QtWidgets import QApplication
+from qutebrowser.qt import machinery
+from qutebrowser.qt.core import PYQT_VERSION_STR
+from qutebrowser.qt.network import QSslSocket
+from qutebrowser.qt.gui import QOpenGLContext, QOffscreenSurface
+from qutebrowser.qt.opengl import QOpenGLVersionProfile
+from qutebrowser.qt.widgets import QApplication
 
 try:
-    from PyQt5.QtWebKit import qWebKitVersion
+    from qutebrowser.qt.webkit import qWebKitVersion
 except ImportError:  # pragma: no cover
     qWebKitVersion = None  # type: ignore[assignment]  # noqa: N816
+try:
+    from qutebrowser.qt.webenginecore import PYQT_WEBENGINE_VERSION_STR
+except ImportError:  # pragma: no cover
+    # QtWebKit
+    PYQT_WEBENGINE_VERSION_STR = None  # type: ignore[assignment]
+
 
 import qutebrowser
-from qutebrowser.utils import log, utils, standarddir, usertypes, message
-from qutebrowser.misc import objects, earlyinit, sql, httpclient, pastebin
+from qutebrowser.utils import (log, utils, standarddir, usertypes, message, resources,
+                               qtutils)
+from qutebrowser.misc import objects, earlyinit, sql, httpclient, pastebin, elf
 from qutebrowser.browser import pdfjs
 from qutebrowser.config import config
-
-try:
-    from qutebrowser.browser.webengine import webenginesettings
-except ImportError:  # pragma: no cover
-    webenginesettings = None  # type: ignore[assignment]
-
+if TYPE_CHECKING:
+    from qutebrowser.config import websettings
 
 _LOGO = r'''
          ______     ,,
@@ -83,11 +75,10 @@ class DistributionInfo:
 
     id: Optional[str]
     parsed: 'Distribution'
-    version: Optional[utils.VersionNumber]
     pretty: str
 
 
-pastebin_url = None
+pastebin_url: Optional[str] = None
 
 
 class Distribution(enum.Enum):
@@ -101,24 +92,21 @@ class Distribution(enum.Enum):
     ubuntu = enum.auto()
     debian = enum.auto()
     void = enum.auto()
-    arch = enum.auto()
+    arch = enum.auto()  # includes rolling-release derivatives
     gentoo = enum.auto()  # includes funtoo
     fedora = enum.auto()
     opensuse = enum.auto()
     linuxmint = enum.auto()
     manjaro = enum.auto()
     kde_flatpak = enum.auto()  # org.kde.Platform
+    neon = enum.auto()
+    nixos = enum.auto()
+    alpine = enum.auto()
+    solus = enum.auto()
 
 
-def distribution() -> Optional[DistributionInfo]:
-    """Get some information about the running Linux distribution.
-
-    Returns:
-        A DistributionInfo object, or None if no info could be determined.
-            parsed: A Distribution enum member
-            version: A Version object, or None
-            pretty: Always a string (might be "Unknown")
-    """
+def _parse_os_release() -> Optional[Dict[str, str]]:
+    """Parse an /etc/os-release file."""
     filename = os.environ.get('QUTE_FAKE_OS_RELEASE', '/etc/os-release')
     info = {}
     try:
@@ -132,40 +120,78 @@ def distribution() -> Optional[DistributionInfo]:
     except (OSError, UnicodeDecodeError):
         return None
 
+    return info
+
+
+def distribution() -> Optional[DistributionInfo]:
+    """Get some information about the running Linux distribution.
+
+    Returns:
+        A DistributionInfo object, or None if no info could be determined.
+            parsed: A Distribution enum member
+            pretty: Always a string (might be "Unknown")
+    """
+    info = _parse_os_release()
+    if info is None:
+        return None
+
     pretty = info.get('PRETTY_NAME', None)
     if pretty in ['Linux', None]:  # Funtoo has PRETTY_NAME=Linux
         pretty = info.get('NAME', 'Unknown')
     assert pretty is not None
 
-    if 'VERSION_ID' in info:
-        version_id = info['VERSION_ID']
-        dist_version: Optional[utils.VersionNumber] = utils.parse_version(version_id)
-    else:
-        dist_version = None
-
     dist_id = info.get('ID', None)
     id_mappings = {
         'funtoo': 'gentoo',  # does not have ID_LIKE=gentoo
+        'artix': 'arch',
         'org.kde.Platform': 'kde_flatpak',
     }
 
-    parsed = Distribution.unknown
+    ids = []
     if dist_id is not None:
+        ids.append(id_mappings.get(dist_id, dist_id))
+    if 'ID_LIKE' in info:
+        ids.extend(info['ID_LIKE'].split())
+
+    parsed = Distribution.unknown
+    for cur_id in ids:
         try:
-            parsed = Distribution[id_mappings.get(dist_id, dist_id)]
+            parsed = Distribution[cur_id]
         except KeyError:
             pass
+        else:
+            break
 
-    return DistributionInfo(parsed=parsed, version=dist_version, pretty=pretty,
-                            id=dist_id)
+    return DistributionInfo(parsed=parsed, pretty=pretty, id=dist_id)
 
 
-def is_sandboxed() -> bool:
-    """Whether the environment has restricted access to the host system."""
-    current_distro = distribution()
-    if current_distro is None:
-        return False
-    return current_distro.parsed == Distribution.kde_flatpak
+def is_flatpak() -> bool:
+    """Whether qutebrowser is running via Flatpak.
+
+    If packaged via Flatpak, the environment is has restricted access to the host
+    system.
+    """
+    return flatpak_id() is not None
+
+
+_FLATPAK_INFO_PATH = '/.flatpak-info'
+
+
+def flatpak_id() -> Optional[str]:
+    """Get the ID of the currently running Flatpak (or None if outside of Flatpak)."""
+    if 'FLATPAK_ID' in os.environ:
+        return os.environ['FLATPAK_ID']
+
+    # 'FLATPAK_ID' was only added in Flatpak 1.2.0:
+    # https://lists.freedesktop.org/archives/flatpak/2019-January/001464.html
+    # but e.g. Ubuntu 18.04 ships 1.0.9.
+    info_file = pathlib.Path(_FLATPAK_INFO_PATH)
+    if not info_file.exists():
+        return None
+
+    parser = configparser.ConfigParser()
+    parser.read(info_file)
+    return parser['Application']['name']
 
 
 def _git_str() -> Optional[str]:
@@ -189,7 +215,7 @@ def _git_str() -> Optional[str]:
         return commit
     # If that fails, check the git-commit-id file.
     try:
-        return utils.read_file('git-commit-id')
+        return resources.read_file('git-commit-id')
     except (OSError, ImportError):
         return None
 
@@ -353,22 +379,39 @@ class ModuleInfo:
         return text
 
 
-MODULE_INFO: Mapping[str, ModuleInfo] = collections.OrderedDict([
-    # FIXME: Mypy doesn't understand this. See https://github.com/python/mypy/issues/9706
-    (name, ModuleInfo(name, *args))  # type: ignore[arg-type, misc]
-    for (name, *args) in
-    [
-        ('sip', ['SIP_VERSION_STR']),
+def _create_module_info() -> Dict[str, ModuleInfo]:
+    packages = [
         ('colorama', ['VERSION', '__version__']),
         ('jinja2', ['__version__']),
         ('pygments', ['__version__']),
         ('yaml', ['__version__']),
         ('adblock', ['__version__'], "0.3.2"),
-        ('PyQt5.QtWebEngineWidgets', []),
-        ('PyQt5.QtWebEngine', ['PYQT_WEBENGINE_VERSION_STR']),
-        ('PyQt5.QtWebKitWidgets', []),
+        ('objc', ['__version__']),
     ]
-])
+
+    if machinery.IS_QT5:
+        packages += [
+            ('PyQt5.QtWebEngineWidgets', []),
+            ('PyQt5.QtWebEngine', ['PYQT_WEBENGINE_VERSION_STR']),
+            ('PyQt5.QtWebKitWidgets', []),
+            ('PyQt5.sip', ['SIP_VERSION_STR']),
+        ]
+    elif machinery.IS_QT6:
+        packages += [
+            ('PyQt6.QtWebEngineCore', ['PYQT_WEBENGINE_VERSION_STR']),
+            ('PyQt6.sip', ['SIP_VERSION_STR']),
+        ]
+    else:
+        raise utils.Unreachable()
+
+    # Mypy doesn't understand this. See https://github.com/python/mypy/issues/9706
+    return {
+        name: ModuleInfo(name, *args)  # type: ignore[arg-type, misc]
+        for (name, *args) in packages
+    }
+
+
+MODULE_INFO: Mapping[str, ModuleInfo] = _create_module_info()
 
 
 def _module_versions() -> Sequence[str]:
@@ -437,13 +480,13 @@ def _pdfjs_version() -> str:
         A string with the version number.
     """
     try:
-        pdfjs_file, file_path = pdfjs.get_pdfjs_res_and_path('build/pdf.js')
+        pdfjs_file, file_path = pdfjs.get_pdfjs_res_and_path(pdfjs.get_pdfjs_js_path())
     except pdfjs.PDFJSNotFound:
         return 'no'
     else:
         pdfjs_file = pdfjs_file.decode('utf-8')
         version_re = re.compile(
-            r"^ *(PDFJS\.version|(var|const) pdfjsVersion) = '(?P<version>[^']+)';$",
+            r"""^ *(PDFJS\.version|(var|const) pdfjsVersion) = ['"](?P<version>[^'"]+)['"];$""",
             re.MULTILINE)
 
         match = version_re.search(pdfjs_file)
@@ -454,47 +497,336 @@ def _pdfjs_version() -> str:
         return '{} ({})'.format(pdfjs_version, file_path)
 
 
-def _chromium_version() -> str:
-    """Get the Chromium version for QtWebEngine.
+def _get_pyqt_webengine_qt_version() -> Optional[str]:
+    """Get the version of the PyQtWebEngine-Qt package.
+
+    With PyQtWebEngine 5.15.3, the QtWebEngine binary got split into its own
+    PyQtWebEngine-Qt PyPI package:
+
+    https://www.riverbankcomputing.com/pipermail/pyqt/2021-February/043591.html
+    https://www.riverbankcomputing.com/pipermail/pyqt/2021-February/043638.html
+
+    PyQtWebEngine 5.15.4 renamed it to PyQtWebEngine-Qt5...:
+    https://www.riverbankcomputing.com/pipermail/pyqt/2021-March/043699.html
+
+    Here, we try to use importlib.metadata to figure out that version number.
+    If PyQtWebEngine is installed via pip, this will give us an accurate answer.
+    """
+    names = (
+        ['PyQt6-WebEngine-Qt6']
+        if machinery.IS_QT6 else
+        ['PyQtWebEngine-Qt5', 'PyQtWebEngine-Qt']
+    )
+
+    for name in names:
+        try:
+            return importlib.metadata.version(name)
+        except importlib.metadata.PackageNotFoundError:
+            log.misc.debug(f"{name} not found")
+
+    return None
+
+
+@dataclasses.dataclass
+class WebEngineVersions:
+
+    """Version numbers for QtWebEngine and the underlying Chromium."""
+
+    webengine: utils.VersionNumber
+    chromium: Optional[str]
+    source: str
+    chromium_security: Optional[str] = None
+    chromium_major: Optional[int] = dataclasses.field(init=False)
+
+    _BASES: ClassVar[Dict[int, str]] = {
+        83: '83.0.4103.122',  # ~2020-06-24
+        87: '87.0.4280.144',  # ~2020-12-02
+        90: '90.0.4430.228',  # 2021-06-22
+        94: '94.0.4606.126',  # 2021-11-17
+        102: '102.0.5005.177',  # ~2022-05-24
+        # (.220 claimed by code, .181 claimed by CHROMIUM_VERSION)
+        108: '108.0.5359.220',  # ~2022-12-23
+        112: '112.0.5615.213',  # ~2023-04-18
+        118: '118.0.5993.220',  # ~2023-10-24
+    }
+
+    _CHROMIUM_VERSIONS: ClassVar[Dict[utils.VersionNumber, Tuple[str, Optional[str]]]] = {
+        # ====== UNSUPPORTED =====
+
+        # Qt 5.12: Chromium 69
+        # (LTS)    69.0.3497.128 (~2018-09-11)
+        #          5.12.10: Security fixes up to 86.0.4240.75 (2020-10-06)
+
+        # Qt 5.13: Chromium 73
+        #          73.0.3683.105 (~2019-02-28)
+        #          5.13.2: Security fixes up to 77.0.3865.120 (2019-10-10)
+
+        # Qt 5.14: Chromium 77
+        #          77.0.3865.129 (~2019-10-10)
+        #          5.14.2: Security fixes up to 80.0.3987.132 (2020-03-03)
+
+        # Qt 5.15: Chromium 80
+        #          80.0.3987.163 (2020-04-02)
+        #          5.15.0: Security fixes up to 81.0.4044.138 (2020-05-05)
+        #          5.15.1: Security fixes up to 85.0.4183.83  (2020-08-25)
+
+        # ====== SUPPORTED =====
+        #                               base         security
+        ## Qt 5.15
+        utils.VersionNumber(5, 15, 2): (_BASES[83], '86.0.4240.183'),  # 2020-11-02
+        utils.VersionNumber(5, 15): (_BASES[87], None),  # >= 5.15.3
+        utils.VersionNumber(5, 15, 3): (_BASES[87], '88.0.4324.150'),  # 2021-02-04
+        # 5.15.4 to 5.15.6: unknown security fixes
+        utils.VersionNumber(5, 15, 7): (_BASES[87], '94.0.4606.61'),  # 2021-09-24
+        utils.VersionNumber(5, 15, 8): (_BASES[87], '96.0.4664.110'),  # 2021-12-13
+        utils.VersionNumber(5, 15, 9): (_BASES[87], '98.0.4758.102'),  # 2022-02-14
+        utils.VersionNumber(5, 15, 10): (_BASES[87], '98.0.4758.102'),  # (?) 2022-02-14
+        utils.VersionNumber(5, 15, 11): (_BASES[87], '98.0.4758.102'),  # (?) 2022-02-14
+        utils.VersionNumber(5, 15, 12): (_BASES[87], '98.0.4758.102'),  # (?) 2022-02-14
+        utils.VersionNumber(5, 15, 13): (_BASES[87], '108.0.5359.124'),  # 2022-12-13
+        utils.VersionNumber(5, 15, 14): (_BASES[87], '113.0.5672.64'),  # 2023-05-02
+        # 5.15.15: unknown security fixes
+        utils.VersionNumber(5, 15, 16): (_BASES[87], '119.0.6045.123'),  # 2023-11-07
+        utils.VersionNumber(5, 15, 17): (_BASES[87], '123.0.6312.58'),  # 2024-03-19
+
+
+        ## Qt 6.2
+        utils.VersionNumber(6, 2): (_BASES[90], '93.0.4577.63'),  # 2021-08-31
+        utils.VersionNumber(6, 2, 1): (_BASES[90], '94.0.4606.61'),  # 2021-09-24
+        utils.VersionNumber(6, 2, 2): (_BASES[90], '96.0.4664.45'),  # 2021-11-15
+        utils.VersionNumber(6, 2, 3): (_BASES[90], '96.0.4664.45'),  # 2021-11-15
+        utils.VersionNumber(6, 2, 4): (_BASES[90], '98.0.4758.102'),  # 2022-02-14
+        # 6.2.5 / 6.2.6: unknown security fixes
+        utils.VersionNumber(6, 2, 7): (_BASES[90], '107.0.5304.110'),  # 2022-11-08
+        utils.VersionNumber(6, 2, 8): (_BASES[90], '111.0.5563.110'),  # 2023-03-21
+
+        ## Qt 6.3
+        utils.VersionNumber(6, 3): (_BASES[94], '99.0.4844.84'),  # 2022-03-25
+        utils.VersionNumber(6, 3, 1): (_BASES[94], '101.0.4951.64'),  # 2022-05-10
+        utils.VersionNumber(6, 3, 2): (_BASES[94], '104.0.5112.81'),  # 2022-08-01
+
+        ## Qt 6.4
+        utils.VersionNumber(6, 4): (_BASES[102], '104.0.5112.102'),  # 2022-08-16
+        utils.VersionNumber(6, 4, 1): (_BASES[102], '107.0.5304.88'),  # 2022-10-27
+        utils.VersionNumber(6, 4, 2): (_BASES[102], '108.0.5359.94'),  # 2022-12-02
+        utils.VersionNumber(6, 4, 3): (_BASES[102], '110.0.5481.78'),  # 2023-02-07
+
+        ## Qt 6.5
+        utils.VersionNumber(6, 5): (_BASES[108], '110.0.5481.104'),  # 2023-02-16
+        utils.VersionNumber(6, 5, 1): (_BASES[108], '112.0.5615.138'),  # 2023-04-18
+        utils.VersionNumber(6, 5, 2): (_BASES[108], '114.0.5735.133'),  # 2023-06-13
+        utils.VersionNumber(6, 5, 3): (_BASES[108], '117.0.5938.63'),  # 2023-09-12
+
+        ## Qt 6.6
+        utils.VersionNumber(6, 6): (_BASES[112], '117.0.5938.63'),  # 2023-09-12
+        utils.VersionNumber(6, 6, 1): (_BASES[112], '119.0.6045.123'),  # 2023-11-07
+        utils.VersionNumber(6, 6, 2): (_BASES[112], '121.0.6167.160'),  # 2024-02-06
+        utils.VersionNumber(6, 6, 3): (_BASES[112], '122.0.6261.128'),  # 2024-03-12
+
+        ## Qt 6.7
+        utils.VersionNumber(6, 7): (_BASES[118], '122.0.6261.128'),  # 2024-03-12
+        utils.VersionNumber(6, 7, 1): (_BASES[118], '124.0.6367.202'),  # ~2024-05-09
+    }
+
+    def __post_init__(self) -> None:
+        """Set the major Chromium version."""
+        if self.chromium is None:
+            self.chromium_major = None
+        else:
+            self.chromium_major = int(self.chromium.split('.')[0])
+
+    def __str__(self) -> str:
+        lines = [f'QtWebEngine {self.webengine}']
+        if self.chromium is not None:
+            lines.append(f'  based on Chromium {self.chromium}')
+        if self.chromium_security is not None:
+            lines.append(f'  with security patches up to {self.chromium_security} (plus any distribution patches)')
+        lines.append(f'  (source: {self.source})')
+        return "\n".join(lines)
+
+    @classmethod
+    def from_ua(cls, ua: 'websettings.UserAgent') -> 'WebEngineVersions':
+        """Get the versions parsed from a user agent.
+
+        This is the most reliable and "default" way to get this information for
+        older Qt versions that don't provide an API for it. However, it needs a
+        fully initialized QtWebEngine, and we sometimes need this information
+        before that is available.
+        """
+        assert ua.qt_version is not None, ua
+        webengine = utils.VersionNumber.parse(ua.qt_version)
+        chromium_inferred, chromium_security = cls._infer_chromium_version(webengine)
+        if ua.upstream_browser_version != chromium_inferred:  # pragma: no cover
+            # should never happen, but let's play it safe
+            log.misc.debug(
+                f"Chromium version mismatch: {ua.upstream_browser_version} (UA) != "
+                f"{chromium_inferred} (inferred)")
+            chromium_security = None
+
+        return cls(
+            webengine=webengine,
+            chromium=ua.upstream_browser_version,
+            chromium_security=chromium_security,
+            source='UA',
+        )
+
+    @classmethod
+    def from_elf(cls, versions: elf.Versions) -> 'WebEngineVersions':
+        """Get the versions based on an ELF file.
+
+        This only works on Linux, and even there, depends on various assumption on how
+        QtWebEngine is built (e.g. that the version string is in the .rodata section).
+
+        On Windows/macOS, we instead rely on from_pyqt, but especially on Linux, people
+        sometimes mix and match Qt/QtWebEngine versions, so this is a more reliable
+        (though hackish) way to get a more accurate result.
+        """
+        webengine = utils.VersionNumber.parse(versions.webengine)
+        chromium_inferred, chromium_security = cls._infer_chromium_version(webengine)
+        if versions.chromium != chromium_inferred:  # pragma: no cover
+            # should never happen, but let's play it safe
+            log.misc.debug(
+                f"Chromium version mismatch: {versions.chromium} (ELF) != "
+                f"{chromium_inferred} (inferred)")
+            chromium_security = None
+
+        return cls(
+            webengine=webengine,
+            chromium=versions.chromium,
+            chromium_security=chromium_security,
+            source='ELF',
+        )
+
+    @classmethod
+    def _infer_chromium_version(
+            cls,
+            pyqt_webengine_version: utils.VersionNumber,
+    ) -> Tuple[Optional[str], Optional[str]]:
+        """Infer the Chromium version based on the PyQtWebEngine version.
+
+        Returns:
+            A tuple of the Chromium version and the security patch version.
+        """
+        chromium_version, security_version = cls._CHROMIUM_VERSIONS.get(
+            pyqt_webengine_version, (None, None))
+        if chromium_version is not None:
+            return chromium_version, security_version
+
+        # 5.15 patch versions change their QtWebEngine version, but no changes are
+        # expected after 5.15.3 and 5.15.[01] are unsupported.
+        assert pyqt_webengine_version != utils.VersionNumber(5, 15, 2)
+
+        # e.g. 5.15.4 -> 5.15
+        # we ignore the security version as that one will have changed from .0
+        # and is thus unknown.
+        minor_version = pyqt_webengine_version.strip_patch()
+        chromium_ver, _security_ver = cls._CHROMIUM_VERSIONS.get(
+            minor_version, (None, None))
+
+        return chromium_ver, None
+
+    @classmethod
+    def from_api(
+        cls,
+        qtwe_version: str,
+        chromium_version: Optional[str],
+        chromium_security: Optional[str] = None,
+    ) -> 'WebEngineVersions':
+        """Get the versions based on the exact versions.
+
+        This is called if we have proper APIs to get the versions easily
+        (Qt 6.2 with PyQt 6.3.1+).
+        """
+        parsed = utils.VersionNumber.parse(qtwe_version)
+        return cls(
+            webengine=parsed,
+            chromium=chromium_version,
+            chromium_security=chromium_security,
+            source='api',
+        )
+
+    @classmethod
+    def from_webengine(
+        cls,
+        pyqt_webengine_qt_version: str,
+        source: str,
+    ) -> 'WebEngineVersions':
+        """Get the versions based on the PyQtWebEngine version.
+
+        This is called if we don't want to fully initialize QtWebEngine (so
+        from_ua isn't possible), we're not on Linux (or ELF parsing failed), but we have
+        a PyQtWebEngine-Qt{,5} package from PyPI, so we could query its exact version.
+        """
+        parsed = utils.VersionNumber.parse(pyqt_webengine_qt_version)
+        chromium, chromium_security = cls._infer_chromium_version(parsed)
+        return cls(
+            webengine=parsed,
+            chromium=chromium,
+            chromium_security=chromium_security,
+            source=source,
+        )
+
+    @classmethod
+    def from_pyqt(cls, pyqt_webengine_version: str, source: str = "PyQt") -> 'WebEngineVersions':
+        """Get the versions based on the PyQtWebEngine version.
+
+        This is the "last resort" if we don't want to fully initialize QtWebEngine (so
+        from_ua isn't possible), we're not on Linux (or ELF parsing failed), and
+        PyQtWebEngine-Qt{5,} isn't available from PyPI.
+
+        Here, we assume that the PyQtWebEngine version is the same as the QtWebEngine
+        version, and infer the Chromium version from that. This assumption isn't
+        generally true, but good enough for some scenarios, especially the prebuilt
+        Windows/macOS releases.
+        """
+        parsed = utils.VersionNumber.parse(pyqt_webengine_version)
+        if utils.VersionNumber(5, 15, 3) <= parsed < utils.VersionNumber(6):
+            # If we land here, we're in a tricky situation where we are forced to guess:
+            #
+            # PyQt 5.15.3 and 5.15.4 from PyPI come with QtWebEngine 5.15.2 (Chromium
+            # 83), not 5.15.3 (Chromium 87). Given that there was no binary release of
+            # QtWebEngine 5.15.3, this is unlikely to change before Qt 6.
+            #
+            # However, at this point:
+            #
+            # - ELF parsing failed
+            #   (so we're likely on macOS or Windows, but not definitely)
+            #
+            # - Getting infos from a PyPI-installed PyQtWebEngine failed
+            #   (so we're either in a PyInstaller-deployed qutebrowser, or a self-built
+            #   or distribution-installed Qt)
+            #
+            # PyQt 5.15.3 and 5.15.4 come with QtWebEngine 5.15.2 (83-based), but if
+            # someone lands here with the last Qt/PyQt installed from source, they might
+            # be using QtWebEngine 5.15.3 (87-based). For now, we play it safe, and only
+            # do this kind of "downgrade" when we know we're using PyInstaller.
+            frozen = hasattr(sys, 'frozen')
+            log.misc.debug(f"PyQt5 >= 5.15.3, frozen {frozen}")
+            if frozen:
+                parsed = utils.VersionNumber(5, 15, 2)
+
+        chromium, chromium_security = cls._infer_chromium_version(parsed)
+
+        return cls(
+            webengine=parsed,
+            chromium=chromium,
+            chromium_security=chromium_security,
+            source=source,
+        )
+
+
+def qtwebengine_versions(*, avoid_init: bool = False) -> WebEngineVersions:
+    """Get the QtWebEngine and Chromium version numbers.
+
+    If we have a parsed user agent, we use it here. If not, we avoid initializing
+    things at all costs (because this gets called early to find out about commandline
+    arguments). Instead, we fall back on looking at the ELF file (on Linux), or, if that
+    fails, use the PyQtWebEngine version.
 
     This can also be checked by looking at this file with the right Qt tag:
     https://code.qt.io/cgit/qt/qtwebengine.git/tree/tools/scripts/version_resolver.py#n41
 
-    Quick reference:
-
-    Qt 5.12: Chromium 69
-    (LTS)    69.0.3497.128 (~2018-09-11)
-             5.12.0: Security fixes up to 70.0.3538.102 (~2018-10-24)
-             5.12.1: Security fixes up to 71.0.3578.94  (2018-12-12)
-             5.12.2: Security fixes up to 72.0.3626.121 (2019-03-01)
-             5.12.3: Security fixes up to 73.0.3683.75  (2019-03-12)
-             5.12.4: Security fixes up to 74.0.3729.157 (2019-05-14)
-             5.12.5: Security fixes up to 76.0.3809.87  (2019-07-30)
-             5.12.6: Security fixes up to 77.0.3865.120 (~2019-09-10)
-             5.12.7: Security fixes up to 79.0.3945.130 (2020-01-16)
-             5.12.8: Security fixes up to 80.0.3987.149 (2020-03-18)
-             5.12.9: Security fixes up to 83.0.4103.97  (2020-06-03)
-             5.12.10: Security fixes up to 86.0.4240.75 (2020-10-06)
-
-    Qt 5.13: Chromium 73
-             73.0.3683.105 (~2019-02-28)
-             5.13.0: Security fixes up to 74.0.3729.157 (2019-05-14)
-             5.13.1: Security fixes up to 76.0.3809.87  (2019-07-30)
-             5.13.2: Security fixes up to 77.0.3865.120 (2019-10-10)
-
-    Qt 5.14: Chromium 77
-             77.0.3865.129 (~2019-10-10)
-             5.14.0: Security fixes up to 77.0.3865.129 (~2019-09-10)
-             5.14.1: Security fixes up to 79.0.3945.117 (2020-01-07)
-             5.14.2: Security fixes up to 80.0.3987.132 (2020-03-03)
-
-    Qt 5.15: Chromium 80
-             80.0.3987.163 (2020-04-02)
-             5.15.0: Security fixes up to 81.0.4044.138 (2020-05-05)
-             5.15.1: Security fixes up to 85.0.4183.83  (2020-08-25)
-
-             5.15.2: Updated to 83.0.4103.122           (~2020-06-24)
-                     Security fixes up to 86.0.4240.183 (2020-11-02)
+    See WebEngineVersions above for a quick reference.
 
     Also see:
 
@@ -502,16 +834,54 @@ def _chromium_version() -> str:
     - https://www.chromium.org/developers/calendar
     - https://chromereleases.googleblog.com/
     """
-    if webenginesettings is None:
-        return 'unavailable'  # type: ignore[unreachable]
+    override = os.environ.get('QUTE_QTWEBENGINE_VERSION_OVERRIDE')
+    if override is not None:
+        return WebEngineVersions.from_pyqt(override, source='override')
 
-    if webenginesettings.parsed_user_agent is None:
-        if 'avoid-chromium-init' in objects.debug_flags:
-            return 'avoided'
+    if machinery.IS_QT6:
+        try:
+            from qutebrowser.qt.webenginecore import (
+                qWebEngineVersion,
+                qWebEngineChromiumVersion,
+            )
+        except ImportError:
+            pass  # Needs QtWebEngine 6.2+ with PyQtWebEngine 6.3.1+
+        else:
+            try:
+                from qutebrowser.qt.webenginecore import (
+                    qWebEngineChromiumSecurityPatchVersion,
+                )
+                chromium_security = qWebEngineChromiumSecurityPatchVersion()
+            except ImportError:
+                chromium_security = None  # Needs QtWebEngine 6.3+
+
+            qtwe_version = qWebEngineVersion()
+            assert qtwe_version is not None
+            return WebEngineVersions.from_api(
+                qtwe_version=qtwe_version,
+                chromium_version=qWebEngineChromiumVersion(),
+                chromium_security=chromium_security,
+            )
+
+    from qutebrowser.browser.webengine import webenginesettings
+
+    if webenginesettings.parsed_user_agent is None and not avoid_init:
         webenginesettings.init_user_agent()
-        assert webenginesettings.parsed_user_agent is not None
 
-    return webenginesettings.parsed_user_agent.upstream_browser_version
+    if webenginesettings.parsed_user_agent is not None:
+        return WebEngineVersions.from_ua(webenginesettings.parsed_user_agent)
+
+    versions = elf.parse_webenginecore()
+    if versions is not None:
+        return WebEngineVersions.from_elf(versions)
+
+    pyqt_webengine_qt_version = _get_pyqt_webengine_qt_version()
+    if pyqt_webengine_qt_version is not None:
+        return WebEngineVersions.from_webengine(
+            pyqt_webengine_qt_version, source='importlib')
+
+    assert PYQT_WEBENGINE_VERSION_STR is not None
+    return WebEngineVersions.from_pyqt(PYQT_WEBENGINE_VERSION_STR)
 
 
 def _backend() -> str:
@@ -519,9 +889,8 @@ def _backend() -> str:
     if objects.backend == usertypes.Backend.QtWebKit:
         return 'new QtWebKit (WebKit {})'.format(qWebKitVersion())
     elif objects.backend == usertypes.Backend.QtWebEngine:
-        webengine = usertypes.Backend.QtWebEngine
-        assert objects.backend == webengine, objects.backend
-        return 'QtWebEngine (Chromium {})'.format(_chromium_version())
+        return str(qtwebengine_versions(
+            avoid_init='avoid-chromium-init' in objects.debug_flags))
     raise utils.Unreachable(objects.backend)
 
 
@@ -561,6 +930,8 @@ def version_info() -> str:
                         platform.python_version()),
         'PyQt: {}'.format(PYQT_VERSION_STR),
         '',
+        str(machinery.INFO),
+        '',
     ]
 
     lines += _module_versions()
@@ -574,7 +945,10 @@ def version_info() -> str:
 
     if objects.qapp:
         style = objects.qapp.style()
-        lines.append('Style: {}'.format(style.metaObject().className()))
+        assert style is not None
+        metaobj = style.metaObject()
+        assert metaobj is not None
+        lines.append('Style: {}'.format(metaobj.className()))
         lines.append('Platform plugin: {}'.format(objects.qapp.platformName()))
         lines.append('OpenGL: {}'.format(opengl_info()))
 
@@ -595,8 +969,8 @@ def version_info() -> str:
         "Imported from {}".format(importpath),
         "Using Python from {}".format(sys.executable),
         "Qt library executable path: {}, data path: {}".format(
-            QLibraryInfo.location(QLibraryInfo.LibraryExecutablesPath),
-            QLibraryInfo.location(QLibraryInfo.DataPath)
+            qtutils.library_path(qtutils.LibraryPath.library_executables),
+            qtutils.library_path(qtutils.LibraryPath.data),
         )
     ]
 
@@ -687,16 +1061,13 @@ def opengl_info() -> Optional[OpenGLInfo]:  # pragma: no cover
     """
     assert QApplication.instance()
 
-    # Some setups can segfault in here if we don't do this.
-    utils.libgl_workaround()
-
     override = os.environ.get('QUTE_FAKE_OPENGL')
     if override is not None:
         log.init.debug("Using override {}".format(override))
         vendor, version = override.split(', ', maxsplit=1)
         return OpenGLInfo.parse(vendor=vendor, version=version)
 
-    old_context = cast(Optional[QOpenGLContext], QOpenGLContext.currentContext())
+    old_context: Optional[QOpenGLContext] = QOpenGLContext.currentContext()
     old_surface = None if old_context is None else old_context.surface()
 
     surface = QOffscreenSurface()
@@ -722,7 +1093,12 @@ def opengl_info() -> Optional[OpenGLInfo]:  # pragma: no cover
         vp.setVersion(2, 0)
 
         try:
-            vf = ctx.versionFunctions(vp)
+            if machinery.IS_QT5:
+                vf = ctx.versionFunctions(vp)
+            else:
+                # Qt 6
+                from qutebrowser.qt.opengl import QOpenGLVersionFunctionsFactory
+                vf: Any = QOpenGLVersionFunctionsFactory.get(vp, ctx)
         except ImportError as e:
             log.init.debug("Importing version functions failed: {}".format(e))
             return None
@@ -731,6 +1107,7 @@ def opengl_info() -> Optional[OpenGLInfo]:  # pragma: no cover
             log.init.debug("Getting version functions failed!")
             return None
 
+        # FIXME:mypy PyQt6-stubs issue?
         vendor = vf.glGetString(vf.GL_VENDOR)
         version = vf.glGetString(vf.GL_VERSION)
 
