@@ -7,7 +7,7 @@
 from unittest import mock
 
 import pytest
-from qutebrowser.qt.core import QRect
+from qutebrowser.qt.core import QRect, QItemSelectionModel
 
 from qutebrowser.completion import completionwidget
 from qutebrowser.completion.models import completionmodel, listcategory
@@ -285,6 +285,23 @@ def test_completion_show(show, rows, quick_complete, completionview, model,
     assert not completionview.isVisible()
 
 
+def test_completion_selection_clear_no_model(completionview):
+    completionview.show()
+    completionview.on_clear_completion_selection()
+    assert completionview.isVisible() is False
+
+
+def test_completion_selection_clear_with_model(completionview, mocker):
+    selmod = mock.Mock(spec=QItemSelectionModel)
+    mocker.patch.object(completionview, "selectionModel", return_value=selmod)
+    completionview.show()
+    completionview.on_clear_completion_selection()
+
+    assert completionview.isVisible() is False
+    selmod.clearSelection.assert_called_once()
+    selmod.clearCurrentIndex.assert_called_once()
+
+
 def test_completion_item_del(completionview, model):
     """Test that completion_item_del invokes delete_cur_item in the model."""
     func = mock.Mock(spec=[])
