@@ -1,28 +1,13 @@
-# vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
-
-# Copyright 2020-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# SPDX-FileCopyrightText: Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
-# This file is part of qutebrowser.
-#
-# qutebrowser is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# qutebrowser is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 import pathlib
 import logging
 import csv
 from typing import Iterable, Tuple
 
-from PyQt5.QtCore import QUrl
+from qutebrowser.qt.core import QUrl
 
 import pytest
 
@@ -327,7 +312,7 @@ def test_whitelist_on_dataset(config_stub, easylist_easyprivacy):
         assert not blockutils.is_whitelisted_url(url)
         config_stub.val.content.blocking.whitelist = []
         assert not blockutils.is_whitelisted_url(url)
-        whitelist_url = url.toString(QUrl.RemovePath) + "/*"
+        whitelist_url = url.toString(QUrl.UrlFormattingOption.RemovePath) + "/*"
         config_stub.val.content.blocking.whitelist = [whitelist_url]
         assert blockutils.is_whitelisted_url(url)
 
@@ -430,3 +415,8 @@ def test_corrupt_cache_handling(ad_blocker, message_mock, caplog):
     assert msg.text == (
         "Reading adblock filter data failed (corrupted data?). "
         "Please run :adblock-update.")
+
+
+def test_resource_type_strings_complete():
+    defined = set(braveadblock._RESOURCE_TYPE_STRINGS) - {None}
+    assert defined == set(ResourceType)
